@@ -49,6 +49,7 @@ class CatalogueEntry:
         self.created_at = vals.get('created_at', 0)
         self.updated_at = vals.get('updated_at', 0)
         self.digest = vals.get('digest', '')
+        self.formatted_title = self.__format_title()
         now = time.time()
         if self.created_at >= (now - TWO_WEEKS_AGO_SECONDS):
             self.freshness = 'Fresh'
@@ -81,7 +82,8 @@ class CatalogueEntry:
             'created_at': self.created_at,
             'updated_at': self.updated_at,
             'freshness': self.freshness,
-            'digest': self.digest
+            'digest': self.digest,
+            'formattedTitle': self.formatted_title
         }
         if self.beqc_url:
             self.for_search['beqcUrl'] = self.beqc_url
@@ -167,11 +169,10 @@ class CatalogueEntry:
                     formatted += self.__format_episodes(formatted, working)
             else:
                 formatted += f"{self.episodes}"
-            return f"{season} {formatted}"
+            return f"{season}{formatted}"
         return season
 
-    @property
-    def formatted_title(self) -> str:
+    def __format_title(self) -> str:
         if self.content_type == 'TV':
             return f"{self.title} {self.__format_tv_meta()}"
         return self.title
@@ -263,22 +264,25 @@ class DatabaseDownloader:
         remote_version = self.__get_remote_catalogue_version()
         if remote_version is None or self.__cached_version is None or remote_version != self.__cached_version:
             logger.info(f"Reloading from {self.__db_url}")
-            r = requests.get(self.__db_url, allow_redirects=True)
-            if r.status_code == 200:
-                logger.info(f"Writing database to {self.__cached_db_file}")
-                with open(self.__cached_db_file, 'wb') as f:
-                    f.write(r.content)
-                if remote_version:
-                    logger.info(f"Writing version {remote_version} to {self.__cached_version_file}")
-                    with open(self.__cached_version_file, 'w') as f:
-                        f.write(remote_version)
+            try:
+                r = requests.get(self.__db_url, allow_redirects=True)
+                if r.status_code == 200:
+                    logger.info(f"Writing database to {self.__cached_db_file}")
+                    with open(self.__cached_db_file, 'wb') as f:
+                        f.write(r.content)
+                    if remote_version:
+                        logger.info(f"Writing version {remote_version} to {self.__cached_version_file}")
+                        with open(self.__cached_version_file, 'w') as f:
+                            f.write(remote_version)
+                    else:
+                        logger.warning(f"No remote version to write")
+                    self.__cached_version = remote_version
+                    logger.info(f"Downloaded {self.__db_url} @ {remote_version}")
+                    return True
                 else:
-                    logger.warning(f"No remote version to write")
-                self.__cached_version = remote_version
-                logger.info(f"Downloaded {self.__db_url} @ {remote_version}")
-                return True
-            else:
-                logger.warning(f"Unable to download catalogue, response is {r.status_code}")
+                    logger.warning(f"Unable to download catalogue, response is {r.status_code}")
+            except:
+                logger.exception(f"Unable to download catalogue, unexpected error")
         else:
             logger.info(f"No reload required {remote_version} vs {self.__cached_version}")
         return False
