@@ -112,6 +112,9 @@ class JRiver(PersistentDevice[JRiverState]):
                             any_update = True
         return any_update
 
+    def load_biquads(self, slot: str, overwrite: bool, inputs: List[int], outputs: List[int], biquads: List[dict]) -> None:
+        raise NotImplementedError()
+
     def load_filter(self, slot: str, entry: CatalogueEntry) -> None:
         def __do_it():
             mc_filts = [make_meta(entry.title, True)] \
@@ -127,7 +130,7 @@ class JRiver(PersistentDevice[JRiverState]):
                 self.__mcws.set_dsp(zone_id, new_config_txt)
                 self._current_state.set_title(slot, entry.formatted_title)
             except Exception as e:
-                self._current_state.slot.last = 'ERRUR'
+                self._current_state.slot.last = 'ERROR'
                 raise e
         self._hydrate_cache_broadcast(__do_it)
 
